@@ -1,7 +1,7 @@
 //! A UTF-8 encoded read-only string using Bytes as storage.
 use std::{borrow, convert::TryFrom, fmt, hash, ops, slice, str};
 
-use crate::Bytes;
+use crate::{Bytes, BytesMut};
 
 /// An immutable UTF-8 encoded string with [`Bytes`] as a storage.
 #[derive(Clone, Default, Eq, PartialOrd, Ord)]
@@ -93,7 +93,7 @@ impl ByteString {
     /// Panics if `at > len`.
     pub fn split_off(&mut self, at: usize) -> ByteString {
         // check str
-        self.split_at(at);
+        let _ = self.split_at(at);
 
         ByteString(self.0.split_off(at))
     }
@@ -123,7 +123,7 @@ impl ByteString {
     /// Panics if `at > len`.
     pub fn split_to(&mut self, at: usize) -> ByteString {
         // check str
-        self.split_at(at);
+        let _ = self.split_at(at);
 
         ByteString(self.0.split_to(at))
     }
@@ -242,7 +242,7 @@ impl TryFrom<Bytes> for ByteString {
     }
 }
 
-impl TryFrom<crate::BytesMut> for ByteString {
+impl TryFrom<BytesMut> for ByteString {
     type Error = str::Utf8Error;
 
     #[inline]
